#include "tasks.h"
#include "schedulability.h"

#include "edf/baker.h"
#include "edf/baruah.h"
#include "edf/gfb.h"
#include "edf/bcl.h"
#include "edf/bcl_iterative.h"
#include "edf/rta.h"
#include "edf/ffdbf.h"
#include "edf/load.h"
#include "edf/gedf.h"

bool GlobalEDF::is_schedulable(const TaskSet &ts,
                               bool check)
{
    if (check)
	{
        if (!(ts.has_only_feasible_tasks() &&
              ts.is_not_overutilized(m)))
            return false;

        if (ts.get_task_count() == 0)
            return true;
    }

    // LL utilization bound on a uniprocessor.
    if (m == 1 && ts.has_only_implicit_deadlines())
    {
        mpq_class util;
        ts.get_utilization(util);
        return util <= 1;
        // If some deadlines are constrained, then it
        // it will be caught by Baruah's test, which
        // generalizes a previous uniprocessor result.
        // We don't especially handle arbitrary deadlines
        // on uniprocessors at this piont.
    }

    // Baker's test can deal with arbitrary deadlines.
    if (BakerGedf(m).is_schedulable(ts, false))
        return true;

    // Baruah's test and the BCL and GFB tests assume constrained deadlines.
    if (ts.has_only_constrained_deadlines())
	    if (GFBGedf(m).is_schedulable(ts, false)
            || BCLGedf(m).is_schedulable(ts, false)
            || BaruahGedf(m).is_schedulable(ts, false)
            || BCLIterativeGedf(m).is_schedulable(ts, false)
            || RTAGedf(m, rta_step).is_schedulable(ts, false)
            || FFDBFGedf(m).is_schedulable(ts, false))
            return true;

    // Load-based test can handle arbitrary deadlines.
    if (LoadGedf(m).is_schedulable(ts, false))
        return true;

    return false;
}
