"""Quantum-based scheduling of sporadic tasks with G-EDF.
"""

from util.quantor   import forall
from util      import us2ms
from quanta    import has_integral_period, quantize_wcet

def account_for_overheads(overheads, tasks, preemptive=True,
                          release_master=False):
    """This differs from PFAIR accounting in two ways:
    1) We account for interrupts as proposed by Uma Devi.
    2) Preemption/migration and scheduling overheads are accounted on a
       per-task basis.
    """
    T = tasks
    qsize = overheads.quantum_size

    # make sure that all periods are multiple of the quantum size
    assert forall(T)(has_integral_period(us2ms(qsize)))

    s   = overheads.sched(T)
    s2  = overheads.sched2(T)
    cxs = overheads.ctxsw(T)
#    print 's:%f    s2:%f    cxs:%f' % (s, s2, cxs)

    preemption_charge  = 0.0
    preemption_charge += 2 * us2ms(s + s2 + cxs)

    if preemptive:
        preemption_charge += 2 * us2ms(overheads.cache_affinity_loss(T))
    
    # IPI ignored, not used by quantum-based EDF
    # release cost ignored, leads to quantum loss

    for i,t in enumerate(T):
        t.cost += preemption_charge

 #   print 'inf1:%f' % T.utilization()

    equant  = float(qsize)
    equant -= overheads.tick(T)
#    print 'equant', equant
    if not release_master:
        equant -= overheads.release(T) * len(T) # all releases every quantum
    if equant > 0:
        quantize_wcet(us2ms(qsize), T, us2ms(equant))
    else:
        # bad, no time left for useful work
        for t in T:
            # hack: make unschedulable
            t.cost = t.period + 1
    return T
