import unittest

import cpptest as cpp

class Test_bounds(unittest.TestCase):

    def setUp(self):
        self.rsi1 = cpp.ResourceSharingInfo(6)

        self.rsi1.add_task(100, 100, 0, 0)
        self.rsi1.add_request(0, 1, 77)

        self.rsi1.add_task(50, 50, 0, 1)
        self.rsi1.add_request(0, 1, 6)

        self.rsi1.add_task(10, 10, 0, 2)
        self.rsi1.add_request(0, 1, 5)

        self.rsi1.add_task(10, 10, 1, 3)
        self.rsi1.add_request(0, 1, 7)

        self.rsi1.add_task(20, 20, 2, 4)
        self.rsi1.add_request(0, 4, 1)

        self.rsi1.add_task(30, 30, 2, 5)
        self.rsi1.add_request(0, 1, 7)


    def test_arrival_blocking_mtx(self):

        c = 1

        res = cpp.task_fair_mutex_bounds(self.rsi1, c)
        self.assertEqual(6 + 7 + 7, res.get_arrival_blocking(0))
        self.assertEqual(5 + 7 + 7, res.get_arrival_blocking(1))
        self.assertEqual(0, res.get_arrival_blocking(2))
        self.assertEqual(0, res.get_arrival_blocking(3))
        self.assertEqual(7 + 7 + 77, res.get_arrival_blocking(4))
        self.assertEqual(0, res.get_arrival_blocking(5))

    def test_arrival_blocking_pf(self):
        c = 1

        res = cpp.phase_fair_rw_bounds(self.rsi1, c)
        self.assertEqual(6 + 7 + 7, res.get_arrival_blocking(0))
        self.assertEqual(5 + 7 + 7, res.get_arrival_blocking(1))
        self.assertEqual(0, res.get_arrival_blocking(2))
        self.assertEqual(0, res.get_arrival_blocking(3))
        self.assertEqual(7 + 7 + 77, res.get_arrival_blocking(4))
        self.assertEqual(0, res.get_arrival_blocking(5))

    def test_arrival_blocking_tf(self):
        c = 1

        res = cpp.task_fair_rw_bounds(self.rsi1, c)
        self.assertEqual(6 + 7 + 7, res.get_arrival_blocking(0))
        self.assertEqual(5 + 7 + 7, res.get_arrival_blocking(1))
        self.assertEqual(0, res.get_arrival_blocking(2))
        self.assertEqual(0, res.get_arrival_blocking(3))
        self.assertEqual(7 + 7 + 77, res.get_arrival_blocking(4))
        self.assertEqual(0, res.get_arrival_blocking(5))



class Test_dedicated_irq(unittest.TestCase):

    def setUp(self):
        self.rsi = cpp.ResourceSharingInfo(16)

        idx = 0
        for cluster, length in zip(range(4), [1, 3, 5, 7]):
            for _ in range(4):
                self.rsi.add_task(100, 100, cluster, idx)
                self.rsi.add_request(0, 1, length)
                idx += 1

        self.rsi_rw = cpp.ResourceSharingInfo(16)

        idx = 0
        for cluster, length in zip(range(4), [1, 3, 5, 7]):
            for _ in range(4):
                self.rsi_rw.add_task(100, 100, cluster, idx)
                self.rsi_rw.add_request_rw(0, 1, length,
                                           cpp.READ if cluster > 0 else cpp.WRITE)
                idx += 1

    def test_global_irq_blocking(self):
        cluster_size = 2
        dedicated_cpu = cpp.NO_CPU
        res = cpp.task_fair_mutex_bounds(self.rsi, cluster_size, dedicated_cpu)

        self.assertEqual(0, res.get_arrival_blocking(15))
        self.assertEqual(7, res.get_blocking_count(15))
        self.assertEqual(2 + 6 + 10 + 7, res.get_blocking_term(15))

        arrival = 2 + 6 + 10 + 14
        self.assertEqual(arrival, res.get_arrival_blocking(0))
        self.assertEqual(15, res.get_blocking_count(0))
        self.assertEqual(arrival + 1 + 6 + 10 + 14, res.get_blocking_term(0))

    def test_dedicated_irq_blocking(self):
        cluster_size = 2
        dedicated_cpu = 0
        res = cpp.task_fair_mutex_bounds(self.rsi, cluster_size, dedicated_cpu)

        self.assertEqual(0, res.get_arrival_blocking(15))
        self.assertEqual(6, res.get_blocking_count(15))
        self.assertEqual(1 + 6 + 10 + 7, res.get_blocking_term(15))

        arrival = 1 + 6 + 10 + 14
        self.assertEqual(arrival, res.get_arrival_blocking(0))
        self.assertEqual(7 + 6, res.get_blocking_count(0))
        self.assertEqual(arrival + 6 + 10 + 14, res.get_blocking_term(0))


    def test_global_irq_tfrw_blocking(self):
        cluster_size = 2
        dedicated_cpu = cpp.NO_CPU

        res = cpp.task_fair_rw_bounds(self.rsi, cluster_size, dedicated_cpu)

        self.assertEqual(0, res.get_arrival_blocking(15))
        self.assertEqual(7, res.get_blocking_count(15))
        self.assertEqual(2 + 6 + 10 + 7, res.get_blocking_term(15))

        arrival = 2 + 6 + 10 + 14
        self.assertEqual(arrival, res.get_arrival_blocking(0))
        self.assertEqual(15, res.get_blocking_count(0))
        self.assertEqual(arrival + 1 + 6 + 10 + 14, res.get_blocking_term(0))



        res = cpp.task_fair_rw_bounds(self.rsi_rw, cluster_size, dedicated_cpu)

        self.assertEqual(0, res.get_arrival_blocking(15))
        self.assertEqual(4, res.get_blocking_count(15))
        self.assertEqual(2 + 5 + 7, res.get_blocking_term(15))

        arrival = 2 + 14
        self.assertEqual(arrival, res.get_arrival_blocking(0))
        self.assertEqual(7, res.get_blocking_count(0))
        # pessimism
        self.assertEqual(arrival + 1 + 14, res.get_blocking_term(0))



    def test_dedicated_irq_tfrw_blocking(self):
        cluster_size = 2
        dedicated_cpu = 0
        res = cpp.task_fair_rw_bounds(self.rsi, cluster_size, dedicated_cpu)

        self.assertEqual(0, res.get_arrival_blocking(15))
        self.assertEqual(6, res.get_blocking_count(15))
        self.assertEqual(1 + 6 + 10 + 7, res.get_blocking_term(15))

        arrival = 1 + 6 + 10 + 14
        self.assertEqual(arrival, res.get_arrival_blocking(0))
        self.assertEqual(7 + 6, res.get_blocking_count(0))
        self.assertEqual(arrival + 6 + 10 + 14, res.get_blocking_term(0))



        res = cpp.task_fair_rw_bounds(self.rsi_rw, cluster_size, dedicated_cpu)

        self.assertEqual(0, res.get_arrival_blocking(15))
        self.assertEqual(2, res.get_blocking_count(15))
        self.assertEqual(1 + 7, res.get_blocking_term(15))

        arrival = 1 + 7
        self.assertEqual(arrival, res.get_arrival_blocking(0))
        self.assertEqual(3, res.get_blocking_count(0))
        # pessimism
        self.assertEqual(arrival + 7, res.get_blocking_term(0))



    def test_global_irq_pfrw_blocking(self):
        cluster_size = 2
        dedicated_cpu = cpp.NO_CPU
        res = cpp.phase_fair_rw_bounds(self.rsi, cluster_size, dedicated_cpu)

        self.assertEqual(0, res.get_arrival_blocking(15))
        self.assertEqual(7, res.get_blocking_count(15))
        self.assertEqual(2 + 6 + 10 + 7, res.get_blocking_term(15))

        arrival = 2 + 6 + 10 + 14
        self.assertEqual(arrival, res.get_arrival_blocking(0))
        self.assertEqual(15, res.get_blocking_count(0))
        self.assertEqual(arrival + 1 + 6 + 10 + 14, res.get_blocking_term(0))


        res = cpp.phase_fair_rw_bounds(self.rsi_rw, cluster_size, dedicated_cpu)

        self.assertEqual(0, res.get_arrival_blocking(15))
        self.assertEqual(2, res.get_blocking_count(15))
        self.assertEqual(1 + 7, res.get_blocking_term(15))

        arrival = 2 + 14
        self.assertEqual(arrival, res.get_arrival_blocking(0))
        self.assertEqual(7, res.get_blocking_count(0))
        # pessimism
        self.assertEqual(arrival + 1 + 14, res.get_blocking_term(0))



    def test_dedicated_irq_pfrw_blocking(self):
        cluster_size = 2
        dedicated_cpu = 0
        res = cpp.phase_fair_rw_bounds(self.rsi, cluster_size, dedicated_cpu)

        self.assertEqual(0, res.get_arrival_blocking(15))
        self.assertEqual(6, res.get_blocking_count(15))
        self.assertEqual(1 + 6 + 10 + 7, res.get_blocking_term(15))

        arrival = 1 + 6 + 10 + 14
        self.assertEqual(arrival, res.get_arrival_blocking(0))
        self.assertEqual(7 + 6, res.get_blocking_count(0))
        self.assertEqual(arrival + 6 + 10 + 14, res.get_blocking_term(0))


        res = cpp.phase_fair_rw_bounds(self.rsi_rw, cluster_size, dedicated_cpu)

        self.assertEqual(0, res.get_arrival_blocking(15))
        self.assertEqual(2, res.get_blocking_count(15))
        self.assertEqual(1 + 7, res.get_blocking_term(15))

        arrival = 1 + 7
        self.assertEqual(arrival, res.get_arrival_blocking(0))
        self.assertEqual(3, res.get_blocking_count(0))
        # pessimism
        self.assertEqual(arrival + 7, res.get_blocking_term(0))


class Test_dpcp_terms(unittest.TestCase):

    def setUp(self):
        self.rsi = cpp.ResourceSharingInfo(4)

        self.rsi.add_task(10, 10, 2, 100)
        self.rsi.add_request(0, 1, 3)

        self.rsi.add_task(25, 25, 3, 200)
        self.rsi.add_request(0, 1, 5)

        self.rsi.add_task(50, 50, 4, 300)
        self.rsi.add_request(0, 1, 7)

        self.rsi.add_task(100, 100, 1, 400)

        self.loc = cpp.ResourceLocality()
        self.loc.assign_resource(0, 1)

    def test_local_blocking(self):
        res = cpp.dpcp_bounds(self.rsi, self.loc)

        self.assertEqual(0, res.get_local_count(0))
        self.assertEqual(0, res.get_local_blocking(0))

        self.assertEqual(0, res.get_local_count(1))
        self.assertEqual(0, res.get_local_blocking(1))

        self.assertEqual(0, res.get_local_count(2))
        self.assertEqual(0, res.get_local_blocking(2))

        self.assertEqual(11 + 5 + 3, res.get_local_count(3))
        self.assertEqual(11 * 3 + 5 * 5 + 3 * 7, res.get_local_blocking(3))

    def test_remote_blocking(self):
        res = cpp.dpcp_bounds(self.rsi, self.loc)

        self.assertEqual(0, res.get_remote_count(3))
        self.assertEqual(0, res.get_remote_blocking(3))

        self.assertEqual(1, res.get_remote_count(0))
        self.assertEqual(7, res.get_remote_blocking(0))

        self.assertEqual(5, res.get_remote_count(1))
        self.assertEqual(4 * 3 + 1 * 7, res.get_remote_blocking(1))

        self.assertEqual(6 + 3, res.get_remote_count(2))
        self.assertEqual(6 * 3 + 3 * 5, res.get_remote_blocking(2))


class Test_mpcp_terms(unittest.TestCase):

    def setUp(self):
        self.rsi = cpp.ResourceSharingInfo(4)

        self.rsi.add_task(10, 10, 2, 100)
        self.rsi.add_request(0, 1, 3)

        self.rsi.add_task(25, 25, 3, 200)
        self.rsi.add_request(0, 1, 5)

        self.rsi.add_task(50, 50, 4, 300)
        self.rsi.add_request(0, 1, 7)

        self.rsi.add_task(100, 100, 1, 400)

        self.loc = cpp.ResourceLocality()
        self.loc.assign_resource(0, 1)


    def test_remote_blocking(self):
        res = cpp.mpcp_bounds(self.rsi, False)

        self.assertEqual(0, res.get_remote_count(3))
        self.assertEqual(0, res.get_remote_blocking(3))

        self.assertEqual(0, res.get_blocking_count(3))
        self.assertEqual(0, res.get_blocking_term(3))

        self.assertEqual(7, res.get_remote_blocking(0))
        self.assertEqual(7, res.get_blocking_term(0))

        self.assertEqual(1 * 7 + (2 + 1) * 3, res.get_remote_blocking(1))
        self.assertEqual(1 * 7 + (2 + 1) * 3, res.get_blocking_term(1))

        self.assertEqual((2 + 1) * 3 + (1 + 1) * 5, res.get_remote_blocking(2))
        self.assertEqual((2 + 1) * 3 + (1 + 1) * 5, res.get_blocking_term(2))


class Test_part_fmlp_terms(unittest.TestCase):

    def setUp(self):
        self.rsi = cpp.ResourceSharingInfo(4)

        self.rsi.add_task(10, 10, 2, 100)
        self.rsi.add_request(0, 1, 3)

        self.rsi.add_task(25, 25, 3, 200)
        self.rsi.add_request(0, 1, 5)

        self.rsi.add_task(50, 50, 4, 300)
        self.rsi.add_request(0, 1, 7)

        self.rsi.add_task(100, 100, 1, 400)

        self.loc = cpp.ResourceLocality()
        self.loc.assign_resource(0, 1)


    def test_fmlp_remote(self):
        res = cpp.part_fmlp_bounds(self.rsi, True)

        self.assertEqual(0, res.get_blocking_count(3))
        self.assertEqual(0, res.get_blocking_term(3))

        self.assertEqual(2, res.get_blocking_count(0))
        self.assertEqual(5 + 7, res.get_blocking_term(0))

        self.assertEqual(2, res.get_blocking_count(1))
        self.assertEqual(3 + 7, res.get_blocking_term(1))

        self.assertEqual(2, res.get_blocking_count(2))
        self.assertEqual(3 + 5, res.get_blocking_term(2))


if __name__ == '__main__':
    unittest.main()
