# math helpers

def gcd(a,b):
    if a == 0:
        return b
    return abs(gcd(b % a, a))

def lcm(*args):
    if not args:
        return 0
    a = args[0]
    for b in args[1:]:
        if not is_integral(a) or not is_integral(b):
            # only well-defined for integers
            raise Exception, \
                "LCM is only well-defined for integers (got: %s, %s)" \
                % (type(a), type(b))
        a = (a / gcd(a,b)) * b
    return a

def ordered_pairs(l):
    for x in xrange(0, len(l)):
        for y in xrange(x + 1, len(l)):
            yield (l[x], l[y])

default_epsilon = 0.000001

def is_multiple(x, y, epsilon=default_epsilon):
    """Is x an integer multiple of y?
       (with at most epsilon error)
    """
    fx = float(x)
    fy = float(y)
    return abs(fx / fy - round(fx / fy)) <= epsilon

def is_integral(x):
    return type(x) == int or type(x) == long

def maybe_int(x):
    if type(x) == float and is_multiple(x, 1):
        return int(x)
    elif type(x) == str:
        try:
            return maybe_int(float(x))
        except ValueError:
            return x
    else:
        return x

def sumn(items, n):
    return sum(sorted(items, reverse=True)[0:n])

def topsum(lst, fun, n):
    """return the sum of the top n items of map(fun, lst)"""
    x = map(fun, lst)
    x.sort(reverse=True)
    return sum(x[0:n])

def vector_xadd(a, b):
    """Do a component-wise destructive add on two lists.
       Semantics: a += b
    """
    for i in xrange(min(len(a), len(b))):
        a[i] += b[i]
    return a

